@@ -8,11 +8,13 @@
 #
 
 
-from reframe.core.exceptions import ReframeSyntaxError
 import reframe.core.namespaces as namespaces
 import reframe.core.parameters as parameters
 import reframe.core.variables as variables
 
+from reframe.core.exceptions import ReframeSyntaxError
+from reframe.core.hooks import HookRegistry
+
 
 class RegressionTestMeta(type):
 
@@ -32,6 +34,7 @@ class RegressionTestMeta(type):
             if isinstance(value, variables.TestVar):
                 # Insert the attribute in the variable namespace
                 self['_rfm_local_var_space'][key] = value
+                value.__set_name__(self, key)
 
                 # Override the regular class attribute (if present)
                 self._namespace.pop(key, None)
@@ -64,9 +67,7 @@ class RegressionTestMeta(type):
             except KeyError as err:
                 try:
                     # Handle variable access
-                    v = self['_rfm_local_var_space'][key]
-                    v.__set_name__(self, key)
-                    return v
+                    return self['_rfm_local_var_space'][key]
 
                 except KeyError:
                     # Handle parameter access
@@ -81,12 +82,10 @@ class RegressionTestMeta(type):
                         # available in the current class' namespace.
                         for b in self['_rfm_bases']:
                             if key in b._rfm_var_space:
-                                v = b._rfm_var_space[key]
-                                v.__set_name__(self, key)
-
                                 # Store a deep-copy of the variable's
                                 # value and return.
-                                self._namespace[key] = v.default_value
+                                v = b._rfm_var_space[key].default_value
+                                self._namespace[key] = v
                                 return self._namespace[key]
 
                         # If 'key' is neither a variable nor a parameter,
@@ -146,27 +145,12 @@ class RegressionTestMeta(type):
         # Set up the hooks for the pipeline stages based on the _rfm_attach
         # attribute; all dependencies will be resolved first in the post-setup
         # phase if not assigned elsewhere
-        hooks = {}
-        fn_with_deps = []
-        for v in namespace.values():
-            if hasattr(v, '_rfm_attach'):
-                for phase in v._rfm_attach:
-                    try:
-                        hooks[phase].append(v)
-                    except KeyError:
-                        hooks[phase] = [v]
+        hooks = HookRegistry.create(namespace)
+        for b in bases:
+            if hasattr(b, '_rfm_pipeline_hooks'):
+                hooks.update(getattr(b, '_rfm_pipeline_hooks'))
 
-            try:
-                if v._rfm_resolve_deps:
-                    fn_with_deps.append(v)
-            except AttributeError:
-                pass
-
-        if fn_with_deps:
-            hooks['post_setup'] = fn_with_deps + hooks.get('post_setup', [])
-
-        cls._rfm_pipeline_hooks = hooks
-        cls._rfm_disabled_hooks = set()
+        cls._rfm_pipeline_hooks = hooks  # HookRegistry(local_hooks)
         cls._final_methods = {v.__name__ for v in namespace.values()
                               if hasattr(v, '_rfm_final')}
 
@@ -236,13 +220,12 @@ class RegressionTestMeta(type):
         return cls._rfm_param_space
 
     def is_abstract(cls):
-        '''Check if the test is an abstract test.
+        '''Check if the class is an abstract test.
 
-        If the parameter space has undefined parameters, the test is considered
-        an abstract test. If that is the case, the length of the parameter
-        space is just 0.
+        This is the case when some parameters are undefined, which results in
+        the length of the parameter space being 0.
 
-        :return: bool indicating wheteher the test is abstract or not
+        :return: bool indicating wheteher the test has undefined parameters.
 
         :meta private:
         '''
